#include <mpi.h>
#include <omp.h>
#include <iostream>
#include <vector>
#include <functional>
#include <unistd.h>
#include "defs.h"

extern void peer2peer_exchange(const std::vector<int> &params);
extern void hip_memcopy(const std::vector<int> &params);
extern int hip_stream(const std::vector<int> &params);
extern void core_affinity(const std::vector<int> &params);
extern void nic_affinity(const std::vector<int> &params);
extern void device_affinity(const std::vector<int> &params);

void do_test(std::function<void(const std::vector<int> &)> fn, const std::vector<int> &params) {
#pragma omp parallel 
    {    
        for (int j = 0; j < 1; j++) {
#pragma omp master
            {
                fn(params);
            }
        }
    }
    MPI_Barrier(MPI_COMM_WORLD);
}

int main(int argc, char **argv)
{
    MPI_Init(&argc, &argv);
    //alarm(70);
    int size, rank;
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    int n = omp_get_max_threads();
    std::cout << ">> rank=" << rank << " nthreads=" << n << std::endl;
    MPI_Barrier(MPI_COMM_WORLD);
    do_test(core_affinity, {rank});
    do_test(device_affinity, {rank});
    do_test(nic_affinity, {rank});
    do_test(hip_stream, {rank});
    const int ngpus = 8;
    for (int distance = 1; distance < ngpus * 2; distance *= 2) {
        do_test(hip_memcopy, {rank, distance, ngpus});
    }
    for (const int use_gpu_memory : {1}) { // {1,0} for both GPU and CPU exch tests
        std::string kind = (use_gpu_memory ? "Device2Device" : "Host2Host");
        print(rank, kind + " MPI exchange, intra-node:");
        for (int distance = 1; distance < ngpus; distance *= 2) {
            do_test(peer2peer_exchange, {rank, size, distance, ngpus, use_gpu_memory});
        }
        if (size > ngpus) {
            print(rank, kind + "MPI exchange, cross-node:");
            for (int distance = 0; distance < ngpus; distance++) {
                do_test(peer2peer_exchange, {rank, size, distance + ngpus, ngpus, use_gpu_memory});
            }
        }
    }
    MPI_Finalize();
    return 0;
}
