#pragma once
#include <iostream>
#include <sstream>
#include <sys/time.h>

struct timer {
  std::stringstream comment;
  std::string name;
  size_t size;
  size_t niters;
  timeval tv[2];
  timer(const std::string& _name, size_t _size, size_t _niters = 1) : name(_name), size(_size), niters(_niters) { gettimeofday(&tv[0], NULL); }
  void start() { gettimeofday(&tv[0], NULL); }
  long time_diff() {
    return ((long)tv[1].tv_sec - (long)tv[0].tv_sec) * 1000000L + (long)tv[1].tv_usec - (long)tv[0].tv_usec;
  }
  ~timer() {
    gettimeofday(&tv[1], NULL);
    long diff = time_diff();
    double GiB_const = (1<<30)/1000000;
    float bw = (float)niters * (float)size / (float)(diff) / GiB_const;
    std::cout << name << ": "
              << "[ "
              << "bandwidth(GiB/s): " << bw << ", "
              << "time(usec): " << diff/niters << ", "
              << "size: " << size;
    if (comment.str().size() != 0) {
        std::cout << ", comment: \"" << comment.str() << "\"";
    }
    std::cout << " "
              << "]" << std::endl;
  }
};

